/***********************************************************************

SlideShowPro-Export-Folder for Photoshop CS2 / CS3 / CS4

This script publishes imagery (from a folder of imagery) and an XML file for SlideShowPro to load. If you need to publish images already loaded in Photoshop, use the "SSP Export Images" script instead.

For installation and usage instructions, see our wiki:
http://wiki.slideshowpro.net/SSPExtras/PhotoshopScripts 

VERSION: 1.0.6

Copyright 2007-9 (c) Dominey Design Inc. All Rights Reserved.

http://slideshowpro.net/

************************************************************************/

//set unit preferences
var strtRulerUnits = app.preferences.rulerUnits;
var strtTypeUnits = app.preferences.typeUnits;
app.preferences.rulerUnits = Units.PIXELS;
app.preferences.typeUnits = TypeUnits.PIXELS;

//Set the cancelled flag
var isCancelled = false;  

//Container var for the sharpening
var sharpLayer;

/////////////////////////////////////////////////////////////////////////////////////////////

//clean up input text with safe html characters
function cleanTxt(txt)
{
	if(txt != null)
	{
		//replace unsafe characters
		var clean = txt;
		clean = clean.replace(/\"/gi,"&quot;");
		clean = clean.replace(/\'/gi,"&quot;");
		clean = clean.replace(/\</gi,"&lt;");
		clean = clean.replace(/\>/gi,"&gt;");
		return clean;
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////

//add underscores and hyphens to file names
function webSafe(txt)
{
	if(txt != null)
	{
		var web = txt;
		var webPrefix = decodeURI(web.substring(0, web.lastIndexOf("."))); // filename body part          
		webPrefix = webPrefix.replace(/ /g, "-"); // space -> '-' (matches save-for-web export)
		webPrefix = webPrefix.replace(/[:\/\\*\?\"\<\>\|]/g, "_");  // '/\:*?"<>|' -> '_'  
		if (webPrefix.length > 27) {
			webPrefix = webPrefix.substring(0,27);
		}
		return webPrefix;
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////

//check for numbers in input
function isNumber(num)
{
	var success;
	
	if(num == null)
	{
		isCancelled = true;
		success = true;
	}
	else if(isFinite(parseInt(num)))
	{
		success = true;
	}
	else
	{
		success = false;
	}
	
	return success;
}

/////////////////////////////////////////////////////////////////////////////////////////////

//create a new slideshow package
function slideShow()
{
	alert("SlideShowPro Export for Photoshop\nTo begin, click OK, then choose the folder containing the images you wish to use.");
	var sourceFolder = Folder.selectDialog("Select folder");
	if(sourceFolder == null)
	{
			isCancelled = true;
	}

	var totalPics = 0;
	if(!isCancelled)
	{
		var items = sourceFolder.getFiles(/\.(jpe|jpg|jpeg|gif|png|tif|tiff|bmp|psd|dng|pict|eps|raw|rw2|crw|cr2)/i);
		totalPics = items.length;
	}
	
	//If no images are open
	if(totalPics == 0)
	{
		alert("No images found in the selected folder. Please try again.");	
	}
	//Otherwise get on with it
	else
	{
		//create a new project folder
		alert("Please create a folder to export content to.");
		var projectFolder = Folder.selectDialog("Create an export folder");
		if(projectFolder == null)
		{
			isCancelled = true;
		}
	
		if(!isCancelled)
		{
			//set the paths for each sub folder
			var glPath = "gallery/";
			var alPath = "gallery/album/";
			var lgPath = "gallery/album/large/";
			var tnPath = "gallery/album/thumb/";
			var tnSmPath = "gallery/album/thumbsm/";
			var fsPath = "gallery/album/fs/";
		
			//Create new xml file
			var myXML = new File(projectFolder + "/images.xml");
		
			//create a new gallery folder
			var gallery = new Folder(projectFolder.absoluteURI + "/" + glPath);
			gallery.create();
		
			//create a new album folder
			var album = new Folder(projectFolder.absoluteURI + "/" + alPath);
			album.create();
		
			//create a new folder to save the images in
			var large = new Folder(projectFolder.absoluteURI + "/" + lgPath);
			large.create();
		
			//create a new folder to save thumbnails in
			var thumbs = new Folder(projectFolder.absoluteURI + "/" + tnPath);
			thumbs.create();
			
			//create a new folder to save small thumbnails in
			var thumbsSm = new Folder(projectFolder.absoluteURI + "/" + tnSmPath);
			thumbsSm.create();
			
			//create a new folder to save the full screen images in
			var fs = new Folder(projectFolder.absoluteURI + "/" + fsPath);
			fs.create();
		
			//set the export options for the images
			var options = new ExportOptionsSaveForWeb();
			options.format = SaveDocumentType.JPEG;
			options.optimized = true;
		}
		
		//get the title
		if(!isCancelled)
		{
			var title = cleanTxt(prompt("Enter an album title","ex: Vacation photos"));
			if(title == null)
			{
				isCancelled = true;
			}
		}
	
		//get the description
		if(!isCancelled)
		{
			var desc = cleanTxt(prompt("Enter an album description","ex: Photos from my summer vacation"));
			if(desc == null)
			{
				isCancelled = true;
			}
		}
		
		//get the max image width from the user
		if(!isCancelled)
		{
			var maxWidth = prompt("Enter the image max width","548");
			while(!isNumber(maxWidth))
			{
				alert("Please enter a number");
				maxWidth = prompt("Enter the image max width","548");
			}
			maxWidth = parseInt(maxWidth);
		}
		
		//get the max image height from the user
		if(!isCancelled)
		{
			var maxHeight = prompt("Enter the image max height","373");
			while(!isNumber(maxHeight))
			{
				alert("Please enter a number");
				maxHeight = prompt("Enter the image max height","373");
			}
			maxHeight = parseInt(maxHeight);
		}
		
		//get the max thumbnail preview width from the user
		if(!isCancelled)
		{
			var maxTnWidth = prompt("Enter the thumbnail preview max width","100");
			while(!isNumber(maxTnWidth))
			{
				alert("Please enter a number");
				maxTnWidth = prompt("Enter the thumbnail preview max width","100");
			}
			maxTnWidth = parseInt(maxTnWidth);
		}
		
		//get the max thumbnail preview height from the user
		if(!isCancelled)
		{
			var maxTnHeight = prompt("Enter the thumbnail preview height","100");
			while(!isNumber(maxTnHeight))
			{
				alert("Please enter a number");
				maxTnHeight = prompt("Enter the thumbnail preview height","100");
			}
			maxTnHeight = parseInt(maxTnHeight);
		}
		
		//get the max thumbnail link width from the user
		if(!isCancelled)
		{
			var maxTnSmWidth = prompt("Enter the thumbnail link width","20");
			while(!isNumber(maxTnSmWidth))
			{
				alert("Please enter a number");
				maxTnSmWidth = prompt("Enter the thumbnail link width","20");
			}
			maxTnSmWidth = parseInt(maxTnSmWidth);
		}
		
		//get the max thumbnail link height from the user
		if(!isCancelled)
		{
			var maxTnSmHeight = prompt("Enter the thumbnail link height","20");
			while(!isNumber(maxTnSmHeight))
			{
				alert("Please enter a number");
				maxTnSmHeight = prompt("Enter the thumbnail link height","20");
			}
			maxTnSmHeight = parseInt(maxTnSmHeight);
		}
		
		//get the max full screen width from the user
		if(!isCancelled)
		{
			var maxFsWidth = prompt("Enter the full screen max width","1024");
			while(!isNumber(maxFsWidth))
			{
				alert("Please enter a number");
				maxTnWidth = prompt("Enter the full screen max width","1024");
			}
			maxFsWidth = parseInt(maxFsWidth);
		}
		
		//get the max full screen height from the user
		if(!isCancelled)
		{
			var maxFsHeight = prompt("Enter the full screen max height","800");
			while(!isNumber(maxFsHeight))
			{
				alert("Please enter a number");
				maxFsHeight = prompt("Enter the full screen max height","800");
			}
			maxFsHeight = parseInt(maxFsHeight);
		}
		
		//get the amount of unsharp mask
		if(!isCancelled)
		{
			var qualAmount = prompt("Enter quality (0-100%)","70");
			while(!isNumber(qualAmount))
			{
				alert("Please enter a number");
				qualAmount = prompt("Enter quality (0-100%)","70");
			}
			qualAmount = parseInt(qualAmount);
			// set quality
			options.quality = qualAmount;
		}
		
		//get the amount of unsharp mask
		if(!isCancelled)
		{
			var sharpAmount = prompt("Enter amount of unsharp mask (0-500%)","100");
			while(!isNumber(sharpAmount))
			{
				alert("Please enter a number");
				sharpAmount = prompt("Enter amount of unsharp mask (0-500%)","100");
			}
			sharpAmount = parseInt(sharpAmount);
		}
		
		if(!isCancelled)
		{
			//Open the new file with the mode set to (w) for write
			myXML.open("w");
			
			//Write the opening XML tags with empty options
			myXML.writeln("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
			myXML.writeln("<gallery>");
			myXML.writeln("<album title=\"" + title + "\" description=\"" + desc + "\" lgpath=\"" + lgPath + "\" tnpath=\"" + tnPath + "\" tnsmpath=\"" + tnSmPath + "\" fspath=\"" + fsPath + "\" tn=\"\">");
						
			//Loop through the open images and get to work
			for (var i = 0; i < totalPics; i++) 
			{     				
				//open each file and give edit access
				var fileRef = new File(items[i]);
				var docRef = app.open(fileRef);
				
				//gather info on the current item
				app.activeDocument = docRef;
				var currWidth = app.activeDocument.width.value;
				var currHeight = app.activeDocument.height.value;
				var caption = cleanTxt(app.activeDocument.info.caption);
				var title = cleanTxt(app.activeDocument.info.headline);
				var filename = webSafe(app.activeDocument.name); 
				
				//set the resample type
				var res = app.activeDocument.resolution;
				var resample = ResampleMethod.BICUBICSHARPER;
				
				//write a new line to the xml file
				myXML.writeln("<img src=\"" + filename + ".jpg" + "\" title=\"" + title + "\" caption=\"" + caption + "\" link=\"\"/>");
				
				
				///////////////////////////////////
				// export: full screen image
				///////////////////////////////////
				
				//perform a fit image action on the current image
				//check the image size against the width constraint
				if(currWidth > maxFsWidth)
				{
					var wPercent = (maxFsWidth / currWidth);
					var newWidth = maxFsWidth + " px";
					var newHeight = Math.ceil(currHeight * wPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currHeight > maxFsHeight)
					{
						var hPercent = (maxFsHeight / currHeight);
						newHeight = maxFsHeight + " px";
						newWidth = Math.ceil(currWidth * hPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				//now check the image size against the height restraint
				else if(currHeight > maxFsHeight)
				{
					var hPercent = (maxFsHeight / currHeight);
					var newHeight = maxFsHeight + " px";
					var newWidth = Math.ceil(currWidth * hPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currWidth > maxFsWidth)
					{
						var wPercent = (maxFsWidth / currWidth);
						newWidth = maxFsWidth = " px";
						newHeight = Math.ceil(currHeight * wPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				
				//Unsharp mask
				if (sharpAmount>0) {
					app.activeDocument.flatten();
					app.activeDocument.selection.selectAll(); 
					app.activeDocument.selection.copy();
					sharpLayer = app.activeDocument.artLayers.add();   
					app.activeDocument.paste();  
				    sharpLayer.applyUnSharpMask(sharpAmount,1,0);
				}
						
				//export the current image, this will strip away all metadata to decrease the filesize
				try
				{
					app.activeDocument.exportDocument(new File(fs + "/" + filename + ".jpg"), ExportType.SAVEFORWEB, options); 
				}
				catch(err)
				{
					var message = "Sorry, image export failed\n";
					message += err;
					alert(message);
				}
								
				// Remove the sharp layer
				if (sharpLayer) {
					sharpLayer.remove();
				} 
				

				///////////////////////////////////
				// export: large image
				///////////////////////////////////
						
				//perform a fit image action on the current image
				//check the image size against the width constraint
				if(currWidth > maxWidth)
				{
					var wPercent = (maxWidth / currWidth);
					var newWidth = maxWidth + " px";
					var newHeight = Math.ceil(currHeight * wPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currHeight > maxHeight)
					{
						var hPercent = (maxHeight / currHeight);
						newHeight = maxHeight + " px";
						newWidth = Math.ceil(currWidth * hPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				//now check the image size against the height restraint
				else if(currHeight > maxHeight)
				{
					var hPercent = (maxHeight / currHeight);
					var newHeight = maxHeight + " px";
					var newWidth = Math.ceil(currWidth * hPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currWidth > maxWidth)
					{
						var wPercent = (maxWidth / currWidth);
						newWidth = maxWidth = " px";
						newHeight = Math.ceil(currHeight * wPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				
				//Unsharp mask
				if (sharpAmount>0) {
					app.activeDocument.selection.selectAll() 
					app.activeDocument.selection.copy()
					sharpLayer = app.activeDocument.artLayers.add();   
					app.activeDocument.paste();  
				    sharpLayer.applyUnSharpMask(sharpAmount,1,0);
				}
		
				//export the current image, this will strip away all metadata to decrease the filesize
				try
				{
					app.activeDocument.exportDocument(new File(large + "/" + filename + ".jpg"), ExportType.SAVEFORWEB, options); 
				}
				catch(err)
				{
					var message = "Sorry, image export failed\n";
					message += err;
					alert(message);
				}
				
				// Remove the sharp layer
				if (sharpLayer) {
					sharpLayer.remove();
				} 
				      
				
				///////////////////////////////////
				// export: thumb image
				///////////////////////////////////
				
				//perform a fit image action on the current image
				//check the image size against the width constraint
				if(currWidth > maxTnWidth)
				{
					var wPercent = (maxTnWidth / currWidth);
					var newWidth = maxTnWidth + " px";
					var newHeight = Math.ceil(currHeight * wPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currHeight > maxTnHeight)
					{
						var hPercent = (maxTnHeight / currHeight);
						newHeight = maxTnHeight + " px";
						newWidth = Math.ceil(currWidth * hPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				//now check the image size against the height restraint
				else if(currHeight > maxTnHeight)
				{
					var hPercent = (maxTnHeight / currHeight);
					var newHeight = maxTnHeight + " px";
					var newWidth = Math.ceil(currWidth * hPercent) + " px";
					app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
			
					currWidth = app.activeDocument.width.value;
					currHeight = app.activeDocument.height.value;
			
					if(currWidth > maxTnWidth)
					{
						var wPercent = (maxTnWidth / currWidth);
						newWidth = maxTnWidth = " px";
						newHeight = Math.ceil(currHeight * wPercent) + " px";
						app.activeDocument.resizeImage(newWidth,newHeight,res,resample);
					}
				}
				
				//Unsharp mask
				if (sharpAmount>0) {
					app.activeDocument.selection.selectAll() 
					app.activeDocument.selection.copy()
					sharpLayer = app.activeDocument.artLayers.add();   
					app.activeDocument.paste();  
				    sharpLayer.applyUnSharpMask(sharpAmount,1,0);
				}
		
				//export the current image, this will strip away all metadata to decrease the filesize
				try
				{
					app.activeDocument.exportDocument(new File(thumbs + "/" + filename + ".jpg"), ExportType.SAVEFORWEB, options);
				}
				catch(err)
				{
					var message = "Sorry, image export failed\n";
					message += err;
					alert(message);
				}
				
				// Remove the sharp layer
				if (sharpLayer) {
					sharpLayer.remove();
				}
				
				
				///////////////////////////////////
				// export: thumb small image
				///////////////////////////////////
				
				//perform a fit image action on the current image
				//check the image size against the width constraint
				var orig_a = currWidth/currHeight;
				var new_a = maxTnSmWidth/maxTnSmHeight;
				var newWidth;
				var newHeight;   
				
				if (orig_a >= new_a) {
				   	newWidth = (maxTnSmHeight * currWidth) / currHeight;
					newHeight = maxTnSmHeight;
		    	} else {
					newWidth = maxTnSmWidth;
					newHeight = (maxTnSmWidth * currHeight) / currWidth;
				}
				
				newWidth += " px";
				newHeight += " px";
	
				app.activeDocument.resizeImage(newWidth,newHeight,res,resample);

				//Unsharp mask
				if (sharpAmount>0) {
					app.activeDocument.selection.selectAll() 
					app.activeDocument.selection.copy()
					sharpLayer = app.activeDocument.artLayers.add();   
					app.activeDocument.paste();  
				    sharpLayer.applyUnSharpMask(sharpAmount,1,0);
				}
				
				// crop to fit   
				var bounds;
				var diff;
				if (newWidth > newHeight) {
					diff = newWidth-maxTnSmWidth; 
					bounds = [diff,0,maxTnSmWidth+diff,maxTnSmHeight];
				} else {
					diff = newHeight-maxTnSmHeight; 
					bounds = [0,diff,maxTnSmWidth,maxTnSmHeight+diff];
				}
				app.activeDocument.crop(bounds);
		
				//export the current image, this will strip away all metadata to decrease the filesize
				try
				{
					app.activeDocument.exportDocument(new File(thumbsSm + "/" + filename + ".jpg"), ExportType.SAVEFORWEB, options);
				}
				catch(err)
				{
					var message = "Sorry, image export failed\n";
					message += err;
					alert(message);
				}
				
                
				///////////////////////////////////
				// done
				///////////////////////////////////

				//close the current file and don't save changes
				app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
			}
			
			//write the closing xml tags
			myXML.writeln("</album>\n</gallery>");
			
			//close the output file
			myXML.close();
			
			//let the user know that the operation completed
			alert("Your slideshow content has been created!")
			
			//reset photoshops preferences
			app.preferences.rulerUnits = strtRulerUnits;
			app.preferences.typeUnits = strtTypeUnits;
		}
		else
		{
			alert("Script cancelled. Please try again.");
		}
	}
}
//create the slideshow source files
slideShow();